#include <windows.h>
#include <stdio.h>

#include <gl/gl.h>
#include <gl/glu.h>

#include "CCamera.h"

CCamera::CCamera()	// Constructeur par dfaut
{
	this->_Mode = __HCCamera__INACTIVE_MODE_;
	this->_ProjectionUpdate	= false;
	this->_pPerspective	= NULL;
	this->_pPosition	= NULL;
	this->_pLookAt		= NULL;
	this->_pUpVector	= NULL;
	this->_pScreen		= NULL;
	this->_Screen		= CIndice2(1,1);
	this->_SetDefault();
}

CCamera::CCamera(CIndice2 &s)	// Constructeur avec valeurs par dfaut ncessitant la rsolution de sortie
{
	this->_Mode = __HCCamera__INACTIVE_MODE_;
	this->_ProjectionUpdate	= false;
	this->_pPerspective	= NULL;
	this->_pPosition	= NULL;
	this->_pLookAt		= NULL;
	this->_pUpVector	= NULL;
	this->_pScreen		= NULL;
	this->_Screen		= s;
	this->_SetDefault();
}

CCamera::CCamera(CIndice2 *s)	// Constructeur avec valeurs par dfaut ncessitant un pointeur vers la rsolution de sortie
{
	this->_Mode = __HCCamera__INACTIVE_MODE_;
	this->_ProjectionUpdate	= false;
	this->_pPerspective	= NULL;
	this->_pPosition	= NULL;
	this->_pLookAt		= NULL;
	this->_pUpVector	= NULL;
	this->_pScreen		= s;
	if (s)
		this->_Screen = *s;
	else
		this->_Screen = CIndice2(1,1);
	this->_SetDefault();
}

CCamera::CCamera(CIndice2 &s, CVector3 &pos, CVector3 &look) // Constructeur avec initialisation de la position et de l'endroit regard
{
	this->_Mode = __HCCamera__INACTIVE_MODE_;
	this->_ProjectionUpdate = false;
	this->_pPerspective	= NULL;
	this->_pPosition	= NULL;
	this->_pLookAt		= NULL;
	this->_pUpVector	= NULL;
	this->_pScreen		= NULL;
	this->_Screen		= s;
	this->_SetDefault();
	this->_Position		= pos;
	this->_LookAt		= look;
}

CCamera::CCamera(CIndice2 *s, CVector3 *pos, CVector3 *look) // Constructeur avec rfrences pour la position et l'endroit regard
{
	this->_Mode = __HCCamera__INACTIVE_MODE_;
	this->_ProjectionUpdate = false;
	this->_pPerspective	= NULL;
	this->_pPosition	= pos;
	this->_pLookAt		= look;
	this->_pUpVector	= NULL;
	this->_pScreen		= s;
	if (s)
		this->_Screen = *s;
	else
		this->_Screen = CIndice2(1,1);
	this->_SetDefault();
	if (pos)
		this->_Position		= *pos;
	if (look)
		this->_LookAt		= *look;
}

CCamera::CCamera(CIndice2 &s, CVector4 &perspec, CVector3 &pos, CVector3 &look,CVector3 &up)	// Constructeur avec initialisation totale
{
	this->_Mode = __HCCamera__INACTIVE_MODE_;
	this->_ProjectionUpdate = false;
	this->_pPerspective	= NULL;
	this->_pPosition	= NULL;
	this->_pLookAt		= NULL;
	this->_pUpVector	= NULL;
	this->_pScreen		= NULL;
	this->_Screen		= s;
	this->_Perspective	= perspec;
	this->_Position		= pos;
	this->_LookAt		= look;
	this->_UpVector		= up;
}

CCamera::CCamera(CIndice2 *s, CVector4 *perspec, CVector3 *pos, CVector3 *look,CVector3 *up)	// Constructeur avec initialisation totale par rfrences
{
	this->_Mode = __HCCamera__INACTIVE_MODE_;
	this->_ProjectionUpdate = false;
	this->_pPerspective	= perspec;
	this->_pPosition	= pos;
	this->_pLookAt		= look;
	this->_pUpVector	= up;
	this->_pScreen		= s;
	if (s)
		this->_Screen = *s;
	else
		this->_Screen = CIndice2(1,1);
	this->_SetDefault();
	if (perspec)
		this->_Perspective	= *perspec;
	if (pos)
		this->_Position		= *pos;
	if (look)
		this->_LookAt		= *look;
	if (up)
		this->_UpVector		= *up;
}

void CCamera::MovePos(CVector3 &v)	// Dplace la position
{
	this->_Position += v;
}

void CCamera::MoveLook(CVector3 &v)	// Dplace le point de regard
{
	this->_LookAt += v;
}

void CCamera::Move(CVector3 &v)	// Dplace la position et le point de regard des mmes coordonnes
{
	this->_Position	+= v;
	this->_LookAt	+= v;
}

void CCamera::Move(CVector3 &vp,CVector3 &vl)	// Dplace la position de vp et le point de regard de vl
{
	this->_Position	+= vp;
	this->_LookAt	+= vl;
}

void CCamera::RotatePos(CVector3 &v)	// Fait tourner la position autour du point de regard
{
	this->_Position -= this->_LookAt;
	this->_Position.Rotate(v);
	this->_Position += this->_LookAt;
}

void CCamera::RotateLook(CVector3 &v)	// Fait tourner le point de regard autour de la position
{
	this->_LookAt -= this->_Position;
	this->_LookAt.Rotate(v);
	this->_LookAt += this->_Position;
}

void CCamera::Draw()	// Envoie les commandes OpenGL pour mettre  jour l'affichage
{
	gluLookAt(this->_Position.xyz[0], this->_Position.xyz[1], this->_Position.xyz[2],
			this->_LookAt.xyz[0], this->_LookAt.xyz[1], this->_LookAt.xyz[2],
			this->_UpVector.xyz[0], this->_UpVector.xyz[1], this->_UpVector.xyz[2]);
	Frustum.CalculateFrustum();	// effectue galement la mise  jour des donnes pour les tests de visibilit
}

void CCamera::Refresh()	// Met  jour les donnes de la camra (cas de donnes manipules par pointeurs)
{
	if (this->_pPerspective)
		this->_Perspective = *this->_pPerspective;
	if (this->_pPosition)
		this->_Position = *this->_pPosition;
	if (this->_pLookAt)
		this->_LookAt = *this->_pLookAt;
	if (this->_pUpVector)
		this->_UpVector = *this->_pUpVector;
}

void CCamera::SetMode(int mode)	// Change le mode de manipulation de la camra
{
	switch (mode)
	{
	case __HCCamera__INACTIVE_MODE_ :			// Mode inactif, aucune modification du point de vue n'est effectue
		this->_ProjectionUpdate = false;
		break;
	case __HCCamera__FPS_MODE_ :				// Mode FPS, on tourne sur soi mme, on se dplace, on strafe
		this->_ProjectionUpdate = false;
		break;
	case __HCCamera__REFERENCE_MODE_ :			// Mode sans controle, les points de position et de vue sont des rfrences
		this->_ProjectionUpdate = true;
		break;
	case __HCCamera__FOLLOW_ROTATE_MODE_ :		// Position fixe, on tourne sur soi mme
		this->_ProjectionUpdate = false;
		break;
	case __HCCamera__FOLLOW_MOVE_MODE_ :		// Position fixe, on dplace le point de vue
		this->_ProjectionUpdate = false;
		break;
	case __HCCamera__FOLLOW_REFERENCE_MODE_ :	// Position fixe, le point de vue est une rfrence
		this->_ProjectionUpdate = true;
		break;
	case __HCCamera__LOOKAT_ROTATE_MODE_ :		// Point de vue fixe, on tourne autour de lui
		this->_ProjectionUpdate = false;
		break;
	case __HCCamera__LOOKAT_MOVE_MODE_ :		// Point de vue fixe, on se dplace
		this->_ProjectionUpdate = false;
		break;
	case __HCCamera__LOOKAT_REFERENCE_MODE_ :	// Point de vue fixe, la position est une rfrence
		this->_ProjectionUpdate = true;
		break;
	}
}

void CCamera::Reshape(CIndice2 &s)	// Envoie la rsolution de sortie  la camra
{
	this->_Screen = s;
	this->_Perspective.xyzw[1] = (float)s.ab[0]/(float)s.ab[1];
}

void CCamera::Reshape(CIndice2 *s)	// Envoie un pointeur sur la rsolution de sortie  la camra
{
	if (s)
	{
		this->_Screen = *s;
		this->_Perspective.xyzw[1] = (float)s->ab[0]/(float)s->ab[1];
	}
}

CVector3 CCamera::GetPos()	// retourne les coordonnes de la position de la camra
{
	return this->_Position;
}

CVector3 CCamera::GetLook()	// retourne les coordonnes de l'endroit o la camra regarde
{
	return this->_LookAt;
}

/*
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(perspective[0].value, perspective[1].value, perspective[2].value, perspective[3].value);
    glGetDoublev(GL_PROJECTION_MATRIX, projection);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(lookat[0].value, lookat[1].value, lookat[2].value, lookat[3].value, lookat[4].value, lookat[5].value, lookat[6].value, lookat[7].value, lookat[8].value);
    glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
*/

void CCamera::_SetDefault()
{
	this->_Perspective	= CVector4(90.0f, (float)this->_Screen.ab[0]/(float)this->_Screen.ab[1], 0.1f, 1000.0f);
	this->_Position		= CVector3(0.0f, 0.0f, 0.0f);
	this->_LookAt		= CVector3(0.0f, 0.0f, 1.0f);
	this->_UpVector		= CVector3(0.0f, 1.0f, 0.0f);
}
