#ifndef __HCCamera__
#define __HCCamera__

#define __HCCamera__INACTIVE_MODE_			0	// Mode inactif, aucune modification du point de vue n'est effectue
#define __HCCamera__FPS_MODE_				10	// Mode FPS, on tourne sur soi mme, on se dplace, on strafe
#define __HCCamera__REFERENCE_MODE_			20	// Mode sans controle, les points de position et de vue sont des rfrences
#define __HCCamera__FOLLOW_ROTATE_MODE_		110	// Position fixe, on tourne sur soi mme
#define __HCCamera__FOLLOW_MOVE_MODE_		120	// Position fixe, on dplace le point de vue
#define __HCCamera__FOLLOW_REFERENCE_MODE_	130	// Position fixe, le point de vue est une rfrence
#define __HCCamera__LOOKAT_ROTATE_MODE_		210	// Point de vue fixe, on tourne autour de lui
#define __HCCamera__LOOKAT_MOVE_MODE_		220	// Point de vue fixe, on se dplace
#define __HCCamera__LOOKAT_REFERENCE_MODE_	230	// Point de vue fixe, la position est une rfrence

#include "CIndice2.h"
#include "CVector3.h"
#include "CVector4.h"
#include "CFrustum.h"

class CCamera {
public:
	CCamera(void);	// Constructeur par dfaut
	CCamera(CIndice2 &s);	// Constructeur avec valeurs par dfaut ncessitant la rsolution de sortie
	CCamera(CIndice2 *s);	// Constructeur avec valeurs par dfaut ncessitant un pointeur vers la rsolution de sortie
	CCamera(CIndice2 &s, CVector3 &pos, CVector3 &look); // Constructeur avec initialisation de la position et de l'endroit regard
	CCamera(CIndice2 *s, CVector3 *pos, CVector3 *look); // Constructeur avec rfrences pour la position et l'endroit regard
	CCamera(CIndice2 &s, CVector4 &perspec, CVector3 &pos, CVector3 &look,CVector3 &up);	// Constructeur avec initialisation totale
	CCamera(CIndice2 *s, CVector4 *perspec, CVector3 *pos, CVector3 *look,CVector3 *up);	// Constructeur avec initialisation totale par rfrences
	void MovePos(CVector3 &v);	// Dplace la position
	void MoveLook(CVector3 &v);	// Dplace le point de regard
	void Move(CVector3 &v);	// Dplace la position et le point de regard des mmes coordonnes
	void Move(CVector3 &vp,CVector3 &vl);	// Dplace la position de vp et le point de regard de vl
	void RotatePos(CVector3 &v);	// Fait tourner la position autour du point de regard
	void RotateLook(CVector3 &v);	// Fait tourner le point de regard autour de la position
	void RotateUp(CVector3 &v);		// Fait tourner la camra autour de son axe de regard
	void Draw(void);	// Envoie les commandes OpenGL pour mettre  jour l'affichage
	void Refresh(void);	// Met  jour les donnes de la camra (cas de donnes manipules par pointeurs)
	void SetMode(int mode);	// Change le mode de manipulation de la camra
	void Reshape(CIndice2 &s);	// Envoie la rsolution de sortie  la camra
	void Reshape(CIndice2 *s);	// Envoie un pointeur sur la rsolution de sortie  la camra
	CVector3 GetPos(void);	// retourne les coordonnes de la position de la camra
	CVector3 GetLook(void);	// retourne les coordonnes de l'endroit o la camra regarde

	CFrustum Frustum;	// le frustum pour les tests de visibilit
private:
	void _SetDefault(void);
	
	int _Mode;	// Mode actuel de la camra
	bool _ProjectionUpdate;	// indique si une mise  jour des donnes est ncessaire
	// donnes de rfrence
	CVector4 *_pPerspective;
	CVector3 *_pPosition, *_pLookAt, *_pUpVector;
	CVector3 *_pAngles;
	CIndice2 *_pScreen;
	// donnes relles utilises pour mettre  jour la matrice projection
	CVector4 _Perspective;
	CVector3 _Position, _LookAt, _UpVector;
	CVector3 _Angles;
	CIndice2 _Screen;
};

#endif // __HCCamera__
