#ifndef __HCFace__
#define __HCFace__

#include "CIndice3.h"
#include "CVector2.h"
#include "CVector3.h"
#include "CVector4.h"

typedef struct
{
	CIndice3 Vertex;
	CIndice3 Normal;
	CIndice3 TexCoord;
}SFace;

typedef struct
{
	CVector3 Vertex[3];
	CVector3 Normal[3];
	CVector2 TexCoord[3];
}SFace3D;

typedef struct
{
	CVector3 *Vertex;
	long nbVertex;
}SPoly3D;

typedef struct
{
	CVector3 v[2];
}SEdge3D;

typedef struct
{
	SEdge3D *Edges;
	long nbEdges;
}SVolume3D;

class CFace
{
public:
	CIndice3 Vertex,Normal,TexCoord;
	CVector3 *pVertices;	// pointeur sur la liste des vertices
	CVector3 *pNormals;		// pointeur sur la liste des normales

	CFace(void);	//	Constructeur par dfaut
	CFace(CIndice3 &v,CIndice3 &n,CIndice3 &t,CVector3 *pV,CVector3 *pN);	// Constructeur  partir de 3 indices + pointeurs sur le tableau contenant les donnes
	CFace(unsigned int v1, unsigned int v2,unsigned int v3, unsigned int n1, unsigned int n2, unsigned int n3, unsigned int t1, unsigned int t2, unsigned int t3,CVector3 *pV,CVector3 *pN);	// Constructeur  partir de 9 indices + pointeurs sur le tableau contenant les donnes
	CFace(CFace &f);	//	Constructeur par duplication
	~CFace(void);	// Destructeur
	CVector4 CalculatePlane(void);	// calcule l'quation du plan dont fait partie la face
/*	float Distance(CVector3 &p);	// Mesure la distance entre une face et un point dans l'espace
	float Distance(CFace &f);	// Mesure la distance entre deux faces
	bool IntersectLTest(CVector3 &p1,CVector3 &p2);	// Teste si une droite passant par p1 et p2 traverse le triangle
	bool IntersectLTest(CVector3 &line);	// Teste si une droite dfinie par le vecteur line traverse le triangle
	bool IntersectSTest(CVector3 &p1,CVector3 &p2);	// Teste si le segment dfini par p1 et p2 traverse le triangle
	bool IntersectSTest(CVector3 &segment);	// Teste si le segment dfini par le vecteur line et sa norme traverse le triangle
	CVector3 IntersectL(CVector3 &p1,CVector3 &p2);	// Retourne les coordonnes de l'intersection du triangle avec la droite passant par p1 et p2
	CVector3 IntersectL(CVector3 &line);	// Retourne les coordonnes de l'intersection du triangle avec la droite dfinie par line
	CVector3 IntersectS(CVector3 &p1,CVector3 &p2);	// Retourne les coordonnes de l'intersection du triangle avec le segment p1, p2
	CVector3 IntersectS(CVector3 &segment);	// Retourne les coordonnes de l'intersection du triangle avec le segment dfini par le vecteur line et sa norme
*/
	void operator = (CFace &v);

	bool operator == (CFace &v);
	bool operator != (CFace &v);
};

// for shadowvolume integration
#define INFINITY	100

class CFace_ShadowVolume
{
public:
	CFace *pFather;	// le pre de la face ombre
	CIndice3 Neighbour;	// index de chaque face voisine  celle-ci
	CVector4 PlaneEquation;	// equation du plan de cette face
	bool isVisible;			// face visible par la source lumineuse

	CFace_ShadowVolume(void);	// constructeur
	~CFace_ShadowVolume(void);	// destructeur
};

#endif // __HCFace__
