#include <float.h>
#include <math.h>

#include "CFastMaths.h"

CFastMaths::CFastMaths()
{
	this->_tdeg = new float[MYMATHS_TABRANGE];
	this->_tsin = new float[MYMATHS_TABRANGE];
	this->_tcos = new float[MYMATHS_TABRANGE];
	this->_trad = new float[MYMATHS_TABRANGE];

	for (int i=0;i<MYMATHS_TABRANGE;i++)	// prparation des tableaux sur l'intervale d'un cercle complet
	{
		this->_trad[i]=(float)(((float)i*2.0f*MYMATHS_PI)/(float)(MYMATHS_TABRANGE));
		if (this->_trad[i]<0.000001f)
			this->_trad[i]=0.0f;
		this->_tdeg[i]=(float)i*(360.0f/(float)(MYMATHS_TABRANGE));
		if (this->_tdeg[i]<0.000001f)
			this->_tdeg[i]=0.0f;
		this->_tsin[i]=(float)sin(this->_trad[i]);
		if (this->_tsin[i]<0.000001f)
			this->_tsin[i]=0.0f;
		this->_tcos[i]=(float)cos(this->_trad[i]);
		if (this->_tcos[i]<0.000001f)
			this->_tcos[i]=0.0f;
	}
}

CFastMaths::~CFastMaths()
{
	delete[] this->_tdeg;
	delete[] this->_tcos;
	delete[] this->_tsin;
	delete[] this->_trad;
}

float CFastMaths::FastCosDeg(float angle)	// retourne le cosinus d'un angle en degrs sans chercher la prcision
{
	while(angle>=360.0f)angle-=360.0f;
	while(angle<0.0f)angle+=360.0f;
	for (int i=0;angle<this->_tdeg[i];i++);
	return this->_tcos[i];
}

float CFastMaths::FastCosRad(float angle)	// retourne le cosinus d'un angle en radians sans chercher la prcision
{
	while(angle>=360.0f)angle-=360.0f;
	while(angle<0.0f)angle+=360.0f;
	for (int i=0;angle<this->_trad[i];i++);
	return this->_tcos[i];
}

float CFastMaths::FastSinDeg(float angle)	// retourne le sinus d'un angle en degrs sans chercher la prcision
{
	while(angle>=360.0f)angle-=360.0f;
	while(angle<0.0f)angle+=360.0f;
	for (int i=0;angle<this->_tdeg[i];i++);
	return this->_tsin[i];
}

float CFastMaths::FastSinRad(float angle)	// retourne le sinus d'un angle en radians sans chercher la prcision
{
	while(angle>=360.0f)angle-=360.0f;
	while(angle<0.0f)angle+=360.0f;
	for (int i=0;angle<this->_trad[i];i++);
	return this->_tsin[i];
}

float CFastMaths::FastAcosDeg(float c)		// retourne l'angle en degrs d'un cosinus
{
	if ((c<-1.0f)||(c>1.0f))
		return 90.0f;
	for (int i=0;c<this->_tcos[i];i++);
	return this->_tdeg[i];
}

float CFastMaths::FastAcosRad(float c)		// retourne l'angle en radians d'un cosinus
{
	if ((c<-1.0f)||(c>1.0f))
		return 90.0f;
	for (int i=0;c<this->_tcos[i];i++);
	return this->_trad[i];
}

float CFastMaths::FastAsinDeg(float s)		// retourne l'angle en degrs d'un cosinus
{
	if ((s<-1.0f)||(s>1.0f))
		return 90.0f;
	for (int i=0;s<this->_tsin[i];i++);
	return this->_tdeg[i];
}

float CFastMaths::FastAsinRad(float s)		// retourne l'angle en radians d'un cosinus
{
	if ((s<-1.0f)||(s>1.0f))
		return 90.0f;
	for (int i=0;s<this->_tsin[i];i++);
	return this->_trad[i];
}

