#include <windows.h>
#include <stdio.h>

#include <gl/gl.h>

#include "CMaterial.h"
#include "ReadLine.h"

#include "Enums.h"
extern CTexture *textures;

CMaterial::CMaterial()	// constructeur par dfaut
{
	this->_nbMaxMat = __HCMaterial__MAX_MATERIALS;
	this->_nbMat = 0;
	this->_LastCalled = -1;
	this->_pMaterial = new SMaterial[__HCMaterial__MAX_MATERIALS];
	CVector4 a(0.2f,0.2f,0.2f,1.0f);
	CVector4 d(1.0f,1.0f,1.0f,1.0f);
	CVector4 e(0.0f,0.0f,0.0f,1.0f);
	CVector4 s(0.0f,0.0f,0.0f,1.0f);
	this->Create("Default",a,d,s,e,0.0f);
}

CMaterial::~CMaterial()	// destructeur
{
	delete[] this->_pMaterial;
}

long CMaterial::Create(char *name,CVector4 a,CVector4 d,CVector4 s,CVector4 e,float sh)	// constructeur  partir de 4 CVector4 + 1 float (pas de texture)
{
	long id = this->GetId(name);
	if (id != 0)
		return id;
	if (this->_nbMat >= this->_nbMaxMat)
		this->_AddMaterials();
	this->_pMaterial[this->_nbMat].Name = CString(name);
	this->_pMaterial[this->_nbMat].Ka = a;
	this->_pMaterial[this->_nbMat].Kd = d;
	this->_pMaterial[this->_nbMat].Ke = e;
	this->_pMaterial[this->_nbMat].Ks = s;
	this->_pMaterial[this->_nbMat].Ns = sh;
	this->_pMaterial[this->_nbMat].nbTex = 0;
	return this->_nbMat++;
}

long CMaterial::Create(char *name,float *all)	// construction  partir d'un tableau de float (pas de texture)
{
	long id = this->GetId(name);
	if (id != 0)
		return id;
	if (this->_nbMat >= this->_nbMaxMat)
		this->_AddMaterials();
	this->_pMaterial[this->_nbMat].Name = CString(name);
	this->_pMaterial[this->_nbMat].Ka = CVector4(all[0] ,all[1] ,all[2] ,all[3]);
	this->_pMaterial[this->_nbMat].Kd = CVector4(all[4] ,all[5] ,all[6] ,all[7]);
	this->_pMaterial[this->_nbMat].Ke = CVector4(all[8] ,all[9] ,all[10],all[11]);
	this->_pMaterial[this->_nbMat].Ks = CVector4(all[12],all[13],all[14],all[15]);
	this->_pMaterial[this->_nbMat].Ns = all[16];
	this->_pMaterial[this->_nbMat].nbTex = 0;
	return this->_nbMat++;
}

long CMaterial::Create(SMaterial *strm)	// contruction  partir des donnes entres dans une structure
{
	long id = this->GetId(strm->Name);
	if (id != 0)
		return id;
	if (this->_nbMat >= this->_nbMaxMat)
		this->_AddMaterials();
	this->_pMaterial[this->_nbMat].Name = strm->Name;
	this->_pMaterial[this->_nbMat].Ka = strm->Ka;
	this->_pMaterial[this->_nbMat].Kd = strm->Kd;
	this->_pMaterial[this->_nbMat].Ke = strm->Ke;
	this->_pMaterial[this->_nbMat].Ks = strm->Ks;
	this->_pMaterial[this->_nbMat].Ns = strm->Ns;
	for (this->_pMaterial[this->_nbMat].nbTex=0;(this->_pMaterial[this->_nbMat].nbTex < __HCMaterial__MAX_TEXTURES)&&(this->_pMaterial[this->_nbMat].nbTex < strm->nbTex);this->_pMaterial[this->_nbMat].nbTex++)
		this->_pMaterial[this->_nbMat].idTex[this->_pMaterial[this->_nbMat].nbTex] = strm->idTex[this->_pMaterial[this->_nbMat].nbTex];
	return this->_nbMat++;
}

bool CMaterial::Load(char *filename)	// charge un fichier matrial
{
	char *buf,buftmp[64];
	unsigned int lenbuf;
	SMaterial mTmp;
	bool materialFound = false;
	FILE *fp = fopen(filename,"rb");

	if (!fp)
		return false;

	while (!feof(fp))
	{
		buf = ReadLine(fp);
		lenbuf = strlen((const char *)buf);
		if (lenbuf > 0)
		{
			sscanf(buf,"%s",buftmp);
			if (!strcmp((const char *)buftmp,"newmtl"))	// on a trouv un nouveau matrial
			{
				if (materialFound)	// si ce n'est pas le premier
					this->Create(&mTmp);	// on enregistre le matrial prcdent
				mTmp.Name = CString(&buf[7]);	// on rcupre le nom de celui que l'on vient de trouver
				mTmp.Ns = 0.0f;	// on initialise ce qui n'a pas
				mTmp.nbTex = 0;	//  t automatiquement initialis
				materialFound = true;	// on indique qu'au moins un matrial a t trouv
			}
			else if (!strcmp((const char *)buftmp,"Ka"))	// on a trouv les valeurs d'une couleur ambiante
				sscanf(&buf[3],"%f%f%f",&mTmp.Ka.xyzw[RED],&mTmp.Ka.xyzw[GREEN],&mTmp.Ka.xyzw[BLUE]);
			else if (!strcmp((const char *)buftmp,"Kd"))	// on a trouv les valeurs d'une couleur diffuse
				sscanf(&buf[3],"%f%f%f",&mTmp.Kd.xyzw[RED],&mTmp.Kd.xyzw[GREEN],&mTmp.Kd.xyzw[BLUE]);
			else if (!strcmp((const char *)buftmp,"Ke"))	// on a trouv les valeurs d'une couleur emissive
				sscanf(&buf[3],"%f%f%f",&mTmp.Ke.xyzw[RED],&mTmp.Ke.xyzw[GREEN],&mTmp.Ke.xyzw[BLUE]);
			else if (!strcmp((const char *)buftmp,"Ks"))	// on a trouv les valeurs d'une couleur specular
				sscanf(&buf[3],"%f%f%f",&mTmp.Ks.xyzw[RED],&mTmp.Ks.xyzw[GREEN],&mTmp.Ks.xyzw[BLUE]);
			else if (!strcmp((const char *)buftmp,"Ns"))	// on a trouv le taux de specular
				sscanf(&buf[3],"%f",&mTmp.Ns);
			else if (!strcmp((const char *)buftmp,"map_Kd"))	// on a trouv une texture
			{
				lenbuf = textures->Load(&buf[7],__HCTexture__TEXPARAM_REPEATALL);
				if (mTmp.nbTex < __HCMaterial__MAX_TEXTURES)
				{
					mTmp.idTex[mTmp.nbTex] = lenbuf;
					mTmp.nbTex++;
				}
			}
		}
		delete[] buf;
	}
	fclose(fp);
	if (materialFound)
		this->Create(&mTmp);
	return materialFound;
}

long CMaterial::GetId(char *name)	// retourne l'id du matrial name
{
	for (long i=0; i < this->_nbMat; i++)
		if (this->_pMaterial[i].Name.Compare(name))
			return i;
	return 0;
}

long CMaterial::GetId(CString name)	// retourne l'id du matrial name
{
	for (long i=0; i < this->_nbMat; i++)
		if (this->_pMaterial[i].Name == name)
			return i;
	return 0;
}

void CMaterial::SetMat(long m)	// fait intervenir le material m dans le rendu
{
	if (m >= this->_nbMat)
		m = 0;
	if (this->_LastCalled == m)
		return;
//	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, this->_pMaterial[m].Ka.xyzw);
//	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, this->_pMaterial[m].Kd.xyzw);
	glColor4fv(this->_pMaterial[m].Kd.xyzw);
//	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, this->_pMaterial[m].Ks.xyzw);
//	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, this->_pMaterial[m].Ke.xyzw);
//	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, this->_pMaterial[m].Ns);
	textures->SetTex(this->_pMaterial[m].nbTex,this->_pMaterial[m].idTex);
	this->_LastCalled = m;
}

void CMaterial::Texture(long m,unsigned int t,unsigned int n)	// intgre la texture t dans le matrial m et la dfinie en position n
{
	if ((m >= this->_nbMat) || (n >= __HCMaterial__MAX_TEXTURES))
		return;
	if (n>this->_pMaterial[m].nbTex)
		n=this->_pMaterial[m].nbTex;
	this->_pMaterial[m].idTex[this->_pMaterial[m].nbTex]=t;
	this->_pMaterial[m].nbTex++;
}

bool CMaterial::AddTexture(long m,unsigned int t)	// ajoute la texture t au matrial m et retourne false si pas de place
{
	if ((m >= this->_nbMat) || (this->_pMaterial[m].nbTex >= __HCMaterial__MAX_TEXTURES))
		return false;
	this->_pMaterial[m].idTex[this->_pMaterial[m].nbTex]=t;
	this->_pMaterial[m].nbTex++;
	return true;
}

void CMaterial::UnTexture(long m)	// retire toutes textures du matrial m
{
	if (m >= this->_nbMat)
		return;
	this->_pMaterial[m].nbTex=0;
}

void CMaterial::UnTexture(long m,unsigned int t)	// retire la texture t du matrial m
{
	if (m >= this->_nbMat)
		return;
	for (unsigned int i=0;i<this->_pMaterial[m].nbTex;i++)
	{
		if (this->_pMaterial[m].idTex[i]==t)
		{
			this->_pMaterial[m].nbTex--;
			for (unsigned int j=i;j<this->_pMaterial[m].nbTex;j++)
				this->_pMaterial[m].idTex[j]=this->_pMaterial[m].idTex[j+1];
		}
	}
}

void CMaterial::_AddMaterials()	// augmente le nombre de nombre de matrials pouvant tre charg de __HCMaterial__NEED_SPACE
{
	this->_nbMaxMat += __HCMaterial__NEED_SPACE;
	SMaterial *tmp = new SMaterial[this->_nbMaxMat];
	for (long i=0;i < this->_nbMat; i++)
		tmp[i] = this->_pMaterial[i];
	delete[] this->_pMaterial;
	this->_pMaterial = tmp;
}
