#ifndef __HCObjet3D__
#define __HCObjet3D__

#include <stdio.h>

#include "CString.h"
#include "CFace.h"

typedef struct
{
public:
	CString Name;
	long nbFaces;
	CFace *pFaces;
	CFace_ShadowVolume *pFaceShadow;
	long MaterialId;
}SGroupe3;

class CObjet3D {
public:
	CObjet3D(void);	//	Cre un dummy
	CObjet3D(CObjet3D &o);	// Cre un objet 3D  partir d'un autre objet (copie)
	CObjet3D(char *filename);	// Cre un objet 3D en chargeant un fichier sur le disque
	CObjet3D(CObjet3D &o,int i);	// Cre un objet 3D  partir d'un groupe d'un autre objet
	~CObjet3D(void);	// Destructeur
	void Draw(void);	// Affiche l'objet en brut
	void Draw(long g);	// Affiche le groupe de l'objet en brut
	void DrawSimple(void);	// Affiche l'objet en brut sans texture ni eclairage
	void DrawSilhouette(CVector3 &source);	// dessine la silhouette de l'objet vue depuis le point source
	void DrawShadowVolume(CVector3 &source);	// dessine le volume d'ombre de l'objet gnr par le point source
	void DrawLists(void);	// Affiche l'objet depuis sa display list
	void DrawList(long g);	// Affiche le groupe de l'objet depuis sa display list
	void GenereLists(void);	// Gnre ou recre la display list de tous les groupes de l'objet
	void RegenList(long g);	// Recree la display list d'un groupe
	void DeleteLists(void);	// Dtruit toutes les display list de l'objet
	void Texture(unsigned int t, long g);
	void InitShadowing(void);	// alloue l'espace ncessaire  la gnration d'ombres

private:
	bool _Load3DS(FILE *fp);	// Charge un objet 3D studio (.3ds)
	bool _LoadLWO(FILE *fp);	// Charge un objet lightwave (.lwo)
	bool _LoadMS3D(FILE *fp);	// Charge un objet milkshape (.ms3d)
	bool _LoadOBJ(FILE *fp);	// Charge un objet wavefront (.obj)
	void _PurifyObject(void);	// supprime les vertex en double dans l'objet
	void _Allocate(void);	// Alloue la mmoire pour stocker l'objet
	void _Destroy(void);	// Vide l'objet
	void _Dummy(void);	// Vide l'objet et cre un dummy  la place (cube de 1m de cot)
	void _DummyMulticolor(void);	// Vide l'objet et cre un dummy multicolor
	void _FindNeighbour(void);	// trouve les 3 faces voisines  chaques faces de l'objet
	void _FindNeighbour(long g);	// trouve les 3 faces voisines  chaques faces du groupe
	void _CalculateFacePlane(void);	// calcule l'quation de plan de chaque face
	SVolume3D * _Silhouette(long g,CVector3 &source);	// gnre la silhouette du groupe g de l'objet vu depuis la position view

	long _nbVertices;
	CVector3 *_pVertices;
	long _nbNormals;
	CVector3 *_pNormals;
	long _nbTexCoords;
	CVector2 *_pTexCoords;
	long _nbGroupes;
	SGroupe3 *_pGroupes;
	unsigned int *_pDisplayLists;
};

#endif // __HCObjet3D__
