#include <float.h>
#include <math.h>

#include "CVector2.h"
#include "CFastMaths.h"

CVector2::CVector2()	// constructeur par dfaut
{
	*this = 0;
}

CVector2::CVector2(CVector2 &v)	// constructeur avec duplication d'un vecteur
{
	*this = v;
}

CVector2::CVector2(float x,float y)	// constructeur avec initialisation du vecteur
{
	this->xy[0] = x;
	this->xy[1] = y;
}

CVector2::~CVector2(void)	// destructeur
{
	// nothing todo
}

void CVector2::Move(CVector2 &v)	// dplace le vecteur
{
	this->xy[0] += v.xy[0];
	this->xy[1] += v.xy[1];
}

void CVector2::Move(float x,float y)	// dplace le vecteur
{
	this->xy[0] += x;
	this->xy[1] += y;
}

void CVector2::RotateR(float angle)	// rotation du vecteur d'un angle donn en radians
{
	float tmp = (float)((this->xy[0]*cos(angle)) - (this->xy[1]*sin(angle)));
	this->xy[1] = (float)((this->xy[0]*sin(angle)) + (this->xy[1]*cos(angle)));
	this->xy[0] = tmp;
}

void CVector2::Rotate(float angle)	// rotation du vecteur d'un angle donn en degrs
{
	angle=(angle/180.0f)*MYMATHS_PI;
	float tmp = (float)((this->xy[0]*cos(angle)) - (this->xy[1]*sin(angle)));
	this->xy[1] = (float)((this->xy[0]*sin(angle)) + (this->xy[1]*cos(angle)));
	this->xy[0] = tmp;
}

void CVector2::Matrix(float *matrix) // traitement du vecteur avec une matrice 3x3
{
	float xtmp = (matrix[0]*this->xy[0]) + (matrix[3]*this->xy[1]);
	this->xy[1] = (matrix[1]*this->xy[0]) + (matrix[4]*this->xy[1]);
	this->xy[0] = xtmp;
}

float CVector2::Length() // longueur du vecteur (aussi appel norme ou magnitude)
{
	return (float)sqrt((this->xy[0]*this->xy[0]) + (this->xy[1]*this->xy[1]));
}

float CVector2::Distance(CVector2 &v)	// Distance entre 2 extrmits de vecteurs
{
	return (float)sqrt((v.xy[0]-this->xy[0])*(v.xy[0]-this->xy[0]) + (v.xy[1]-this->xy[1])*(v.xy[1]-this->xy[1]));
}

float CVector2::Dot(CVector2 &v)	// Produit scalaire
{
	return ((this->xy[0]*v.xy[0]) + (this->xy[1]*v.xy[1]));
}

CVector2 CVector2::Cross(CVector2 &v)	// Normale  2 vecteurs (produit vectoriel)
{
	return CVector2(this->xy[1]*v.xy[0] - v.xy[1]*this->xy[0], this->xy[0]*v.xy[1]-this->xy[1]*v.xy[0]);
}

CVector2 CVector2::Linterpolate (CVector2 &u, CVector2 &v, float x)	// Fonction d'interpolation linaire entre 2 vecteurs
{
	return u * (1 - x) + v * x;
}
/*
CVector2 CVector2::Cinterpolate (CVector2 &u0, CVector2 &u1, CVector2 &u2, CVector2 &u3, float x)	// Fonction d'interpolation circulaire
{
	CVector2 p = (u3 - u2) - (u0 - u1);
	CVector2 q = (u0 - u1) - p;
	CVector2 r = u2 - u0;
	return (((p * x + q) * x + r) * x + u1);
}
*/
float CVector2::AngleRad(CVector2 &v2)	// retourne l'angle en radians entre le vecteur *this et le vecteur v2
{
    float angle = (float)acos( this->Dot(v2) / (this->Length() * v2.Length()) );

    if(_isnan(angle))
        return 0.0f;

    return angle;
}

float CVector2::AngleDeg(CVector2 &v2)	// retourne l'angle en degrs entre le vecteur *this et le vecteur v2
{                           
    float angle = (float)(180.0f * (acos( this->Dot(v2) / this->Length() * v2.Length() ) / MYMATHS_PI));

    if(_isnan(angle))
        return 0;
    
    return angle;
}

void CVector2::Normalize()	// normalise le vecteur
{
	float l = Length();

	if (l > 0.0f)
	{
		this->xy[0] /= l;
		this->xy[1] /= l;
	}
}

CVector2 CVector2::Normalized()	// retourne la norme du vecteur
{
	float l = Length();

	if (l > 0.0f)
	{
		return CVector2(this->xy[0]/l,this->xy[1]/l);
	}
	return CVector2(0,1);
}

CVector2 CVector2::ClosestPointOnLine(CVector2 &vA, CVector2 &vB)	// retourne les coordonne du point le plus proche de *this sur la droite passant par vA et vB
{
	return (((vB-vA) * ProjectionOnLine(vA, vB)) + vA);
}

CVector2 CVector2::ClosestPointOnSegment(CVector2 &vA, CVector2 &vB)	// retourne les coordonne du point le plus proche de *this sur le segment vA,vB
{
	float factor = ProjectionOnLine(vA, vB);
	if (factor<=0.0f)
		return vA;
	if (factor>=1.0f)
		return vB;
	return (((vB-vA) * factor) + vA);
}

float CVector2::ProjectionOnLine(CVector2 &vA, CVector2 &vB)	// retourne le facteur de la projection de *this sur la droite passant par vA et vB
{
	CVector2 v = vB - vA;
	return v.Dot(*this - vA) / v.Dot(v);
}

CVector2 CVector2::Vector(CVector2 &vP)	// retourne le vecteur entre *this et vP
{
    return CVector2(vP.xy[0]-this->xy[0], vP.xy[1]-this->xy[1]);
}

//////////////////////////////
//	Les oprateurs simples	//
//////////////////////////////

void CVector2::operator = (CVector2 &v)
{
	this->xy[0] = v.xy[0];
	this->xy[1] = v.xy[1];
}

void CVector2::operator = (float i)
{
	this->xy[0] = i;
	this->xy[1] = i;
}

CVector2 CVector2::operator + (CVector2 &v)
{
	return CVector2(this->xy[0]+v.xy[0], this->xy[1]+v.xy[1]);
}

CVector2 CVector2::operator + (float i)
{
	return CVector2(this->xy[0]+i, this->xy[1]+i);
}

CVector2 CVector2::operator - (CVector2 &v)
{
	return CVector2(this->xy[0]-v.xy[0], this->xy[1]-v.xy[1]);
}

CVector2 CVector2::operator - (float i)
{
	return CVector2(this->xy[0]-i, this->xy[1]-i);
}

CVector2 CVector2::operator * (CVector2 &v)
{
	return CVector2(this->xy[0]*v.xy[0], this->xy[1]*v.xy[1]);
}

CVector2 CVector2::operator * (float i)
{
	return CVector2(this->xy[0]*i, this->xy[1]*i);
}

CVector2 CVector2::operator / (CVector2 &v)
{
	return CVector2(this->xy[0]/v.xy[0], this->xy[1]/v.xy[1]);
}

CVector2 CVector2::operator / (float i)
{
	return CVector2(this->xy[0]/i, this->xy[1]/i);
}

//////////////////////////////
//	Les oprateurs floats	//
//////////////////////////////

void CVector2::operator += (CVector2 &v)
{
	this->xy[0] += v.xy[0];
	this->xy[1] += v.xy[1];
}

void CVector2::operator += (float i)
{
	this->xy[0] += i;
	this->xy[1] += i;
}

void CVector2::operator -= (CVector2 &v)
{
	this->xy[0] -= v.xy[0];
	this->xy[1] -= v.xy[1];
}

void CVector2::operator -= (float i)
{
	this->xy[0] -= i;
	this->xy[1] -= i;
}

void CVector2::operator *= (CVector2 &v)
{
	this->xy[0] *= v.xy[0];
	this->xy[1] *= v.xy[1];
}

void CVector2::operator *= (float i)
{
	this->xy[0] *= i;
	this->xy[1] *= i;
}

void CVector2::operator /= (CVector2 &v)
{
	this->xy[0] /= v.xy[0];
	this->xy[1] /= v.xy[1];
}

void CVector2::operator /= (float i)
{
	this->xy[0] /= i;
	this->xy[1] /= i;
}

//////////////////////////////
//	Les oprateurs de test	//
//////////////////////////////

bool CVector2::operator == (CVector2 &v)
{
	return ((this->xy[0]==v.xy[0])&&(this->xy[1]==v.xy[1]));
}

bool CVector2::operator != (CVector2 &v)
{
	return ((this->xy[0]!=v.xy[0])||(this->xy[1]!=v.xy[1]));
}

