#ifndef __HCVector2__
#define __HCVector2__

typedef struct
{
	float xy[2];
}SVector2;

class CVector2 {
public:
	float xy[2];

    CVector2(void);	// constructeur par dfaut
    CVector2(CVector2 &v);	// constructeur avec duplication d'un vecteur
    CVector2(float x,float y);	// constructeur avec initialisation du vecteur
    ~CVector2(void);	// destructeur
    void Move(CVector2 &v);	// dplace le vecteur
    void Move(float x,float y);	// dplace le vecteur
    void RotateR(float angle);	// rotation du vecteur d'un angle donn en radians
    void Rotate(float angle);	// rotation du vecteur d'un angle donn en degrs
    void Matrix(float *matrix); // traitement du vecteur avec une matrice 3x3
    float Length(void); // longueur du vecteur (aussi appel norme ou magnitude)
    float Distance(CVector2 &v);	// Distance entre 2 extrmits de vecteurs
    float Dot(CVector2 &v);	// Produit scalaire
    CVector2 Cross(CVector2 &v);	// Normale  2 vecteurs (produit vectoriel)
    CVector2 Linterpolate(CVector2 &u, CVector2 &v, float x);	// Fonction d'interpolation linaire entre 2 vecteurs
//    CVector2 Cinterpolate(CVector2 &u0, CVector2 &u1, CVector2 &u2, CVector2 &u3, float x);	// Fonction d'interpolation circulaire
    float AngleRad(CVector2 &v);	// retourne l'angle en radians entre le vecteur *this et le vecteur v2
    float AngleDeg(CVector2 &v2);	// retourne l'angle en degrs entre le vecteur *this et le vecteur v2
    void Normalize(void);	// normalise le vecteur
    CVector2 Normalized(void);	// retourne la norme du vecteur
    CVector2 ClosestPointOnLine(CVector2 &vA, CVector2 &vB);	// retourne les coordonne du point le plus proche de *this sur la droite passant par vA et vB
    CVector2 ClosestPointOnSegment(CVector2 &vA, CVector2 &vB);	// retourne les coordonne du point le plus proche de *this sur le segment vA,vB
    float ProjectionOnLine(CVector2 &vA, CVector2 &vB);	// retourne le facteur de la projection de *this sur la droite passant par vA et vB
    CVector2 Vector(CVector2 &vP);	// retourne le vecteur entre *this et vP

    void operator = (CVector2 &v);
    void operator = (float i);
    CVector2 operator + (CVector2 &v);
    CVector2 operator + (float i);
    CVector2 operator - (CVector2 &v);
    CVector2 operator - (float i);
    CVector2 operator * (CVector2 &v);
    CVector2 operator * (float i);
    CVector2 operator / (CVector2 &v);
    CVector2 operator / (float i);

    void operator += (CVector2 &v);
    void operator += (float i);
    void operator -= (CVector2 &v);
    void operator -= (float i);
    void operator *= (CVector2 &v);
    void operator *= (float i);
    void operator /= (CVector2 &v);
    void operator /= (float i);

    bool operator == (CVector2 &v);
    bool operator != (CVector2 &v);

    operator const float *() const { return xy; }
    float &operator[](unsigned int index){ return *(((float *) xy) + index); }
};

#endif // __HCVector2__
