#include <float.h>
#include <math.h>

#include "CVector2.h"
#include "CVector4.h"
#include "CVector3.h"
#include "CFastMaths.h"

CVector3::CVector3()	// constructeur par dfaut
{
	*this = 0;
}

CVector3::CVector3(CVector3 &v)	// constructeur avec duplication d'un vecteur
{
	*this = v;
}

#ifdef __HCVector2__
CVector3::CVector3(CVector2 &v,float z)	// constructeur d'un CVector3  partir d'un CVector2 + coordonne Z
{
	this->xyz[0] = v.xy[0];
	this->xyz[1] = v.xy[1];
	this->xyz[2] = z;
}

CVector3::CVector3(float z,CVector2 &v)	// constructeur d'un CVector3  partir d'un coordonne Z + CVector2
{
	this->xyz[0] = v.xy[0];
	this->xyz[1] = v.xy[1];
	this->xyz[2] = z;
}
#endif

#ifdef __HCVector4__
CVector3::CVector3 (CVector4 &v)	// constructeur d'un CVector3  partir d'un CVector4
{
	this->xyz[0] = v.xyzw[0];
	this->xyz[1] = v.xyzw[1];
	this->xyz[2] = v.xyzw[2];
}
#endif

CVector3::CVector3(float x,float y,float z)	// constructeur avec initialisation du vecteur
{
	this->xyz[0] = x;
	this->xyz[1] = y;
	this->xyz[2] = z;
}

CVector3::~CVector3()	// destructeur
{
	// nothing todo
}

void CVector3::Move(CVector3 &v)	// dplace le vecteur
{
	*this += v;
}

void CVector3::Move(float x,float y,float z)	// dplace le vecteur
{
	this->xyz[0] += x;
	this->xyz[1] += y;
	this->xyz[2] += z;
}

void CVector3::RotateXr(float angle)	// rotation du vecteur sur l'axe X d'un angle donn en radians
{
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyz[1]*ca-this->xyz[2]*sa;
	this->xyz[2]=this->xyz[1]*sa+this->xyz[2]*ca;
	this->xyz[1]=tmp;
}

void CVector3::RotateYr(float angle)	// rotation du vecteur sur l'axe Y d'un angle donn en radians
{
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyz[0]*ca+this->xyz[2]*sa;
	this->xyz[2]=this->xyz[2]*ca-this->xyz[0]*sa;
	this->xyz[0]=tmp;
}

void CVector3::RotateZr(float angle)	// rotation du vecteur sur l'axe Z d'un angle donn en radians
{
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyz[0]*ca-this->xyz[1]*sa;
	this->xyz[1]=this->xyz[0]*sa+this->xyz[1]*ca;
	this->xyz[0]=tmp;
}

void CVector3::RotateX(float angle)	// rotation du vecteur sur l'axe X d'un angle donn en degrs
{
	angle=(angle/180.0f)*MYMATHS_PI;
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyz[1]*ca-this->xyz[2]*sa;
	this->xyz[2]=this->xyz[1]*sa+this->xyz[2]*ca;
	this->xyz[1]=tmp;
}

void CVector3::RotateY(float angle)	// rotation du vecteur sur l'axe Y d'un angle donn en degrs
{
	angle=(angle/180.0f)*MYMATHS_PI;
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyz[0]*ca+this->xyz[2]*sa;
	this->xyz[2]=this->xyz[2]*ca-this->xyz[0]*sa;
	this->xyz[0]=tmp;
}

void CVector3::RotateZ(float angle)	// rotation du vecteur sur l'axe Z d'un angle donn en degrs
{
	angle=(angle/180.0f)*MYMATHS_PI;
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyz[0]*ca-this->xyz[1]*sa;
	this->xyz[1]=this->xyz[0]*sa+this->xyz[1]*ca;
	this->xyz[0]=tmp;
}

void CVector3::Rotate(CVector3 &angle)	// rotation du vecteur sur les 3 axes XYZ d'un angle donn en degrs
{
	CVector3 angled = (angle/180.0f)*MYMATHS_PI;
	float cx = (float)cos(angled.xyz[0]);
	float sx = (float)sin(angled.xyz[0]);
	float cy = (float)cos(angled.xyz[1]);
	float sy = (float)sin(angled.xyz[1]);
	float cz = (float)cos(angled.xyz[2]);
	float sz = (float)sin(angled.xyz[2]);

	float tmpx = this->xyz[0]*cy*cz - this->xyz[1]*(cx*sz + cz*sy*sx) + this->xyz[2]*(sz*sx + cz*sy*cx);
	float tmpy = this->xyz[1]*(cx*cz + sx*sy*sz) - this->xyz[2]*(sx*cz - sz*sy*cx) + this->xyz[0]*sz*cy;
	this->xyz[2] = this->xyz[2]*cx*cy - this->xyz[0]*sy + this->xyz[1]*cy*sx;
	this->xyz[1] = tmpy;
	this->xyz[0] = tmpx;
}

void CVector3::Matrix(float *matrix) // traitement du vecteur avec une matrice 4x4
{
	float xtmp = (matrix[0]*this->xyz[0]) + (matrix[4]*this->xyz[1]) + (matrix[8]*this->xyz[2]) + matrix[12];
	float ytmp = (matrix[1]*this->xyz[0]) + (matrix[5]*this->xyz[1]) + (matrix[9]*this->xyz[2]) + matrix[13];
	this->xyz[2] = (matrix[2]*this->xyz[0]) + (matrix[6]*this->xyz[1]) + (matrix[10]*this->xyz[2]) + matrix[14];
	this->xyz[1] = ytmp;
	this->xyz[0] = xtmp;
}

float CVector3::Length() // longueur du vecteur (aussi appel norme ou magnitude)
{
	return (float) sqrt((this->xyz[0]*this->xyz[0]) + (this->xyz[1]*this->xyz[1]) + (this->xyz[2]*this->xyz[2]));
}

float CVector3::Distance(CVector3 &v)	// Distance entre 2 extrmits de vecteurs
{
	return (float)sqrt((v.xyz[0]-this->xyz[0])*(v.xyz[0]-this->xyz[0]) + (v.xyz[1]-this->xyz[1])*(v.xyz[1]-this->xyz[1]) + (v.xyz[2]-this->xyz[2])*(v.xyz[2]-this->xyz[2]));
}

float CVector3::Dot(CVector3 &v)	// Produit scalaire
{
	return ((this->xyz[0]*v.xyz[0]) + (this->xyz[1]*v.xyz[1]) + (this->xyz[2]*v.xyz[2]));
}

CVector3 CVector3::Cross(CVector3 &v)	// Normale  2 vecteurs (produit vectoriel)
{
	return CVector3(this->xyz[1]*v.xyz[2]-v.xyz[1]*this->xyz[2], this->xyz[2]*v.xyz[0]-this->xyz[0]*v.xyz[2], this->xyz[0]*v.xyz[1]-this->xyz[1]*v.xyz[0]);
}

CVector3 CVector3::Linterpolate (CVector3 &u, CVector3 &v, float x)	// Fonction d'interpolation linaire entre 2 vecteurs
{
	return u * (1 - x) + v * x;
}
/*
CVector3 CVector3::Cinterpolate (CVector3 &u0, CVector3 &u1, CVector3 &u2, CVector3 &u3, float x)	// Fonction d'interpolation circulaire
{
	CVector3 p = (u3 - u2) - (u0 - u1);
	CVector3 q = (u0 - u1) - p;
	CVector3 r = u2 - u0;
	return (((p * x + q) * x + r) * x + u1);
}
*/
float CVector3::AngleRad(CVector3 &v2)	// retourne l'angle en radians entre le vecteur *this et le vecteur v2
{                           
    float angle = (float)acos( this->Dot(v2) / this->Length() * v2.Length() );

    if(_isnan(angle))
        return 0;
    
    return angle;
}

float CVector3::AngleDeg(CVector3 &v2)	// retourne l'angle en degrs entre le vecteur *this et le vecteur v2
{                           
    float angle = (float)(180.0f * (acos( this->Dot(v2) / this->Length() * v2.Length() ) / MYMATHS_PI));

    if(_isnan(angle))
        return 0;
    
    return angle;
}

void CVector3::Normalize()	// normalise le vecteur
{
  float l = Length();

  if (l > 0.0f)
	*this /= l;
}

CVector3 CVector3::Normalized()	// retourne la norme de *this
{
	float l = Length();

	if (l > 0.0f)
	{
		*this /= l;
		return *this;
	}
	return CVector3(0,1,0);
}

CVector3 CVector3::ClosestPointOnLine(CVector3 &vA, CVector3 &vB)	// retourne les coordonne du point le plus proche de *this sur la droite passant par vA et vB
{
	return (((vB-vA) * ProjectionOnLine(vA, vB)) + vA);
}

CVector3 CVector3::ClosestPointOnSegment(CVector3 &vA, CVector3 &vB)	// retourne les coordonne du point le plus proche de *this sur le segment vA,vB
{
	float factor = ProjectionOnLine(vA, vB);
	if (factor<=0.0f)
		return vA;
	if (factor>=1.0f)
		return vB;
	return (((vB-vA) * factor) + vA);
}

float CVector3::ProjectionOnLine(CVector3 &vA, CVector3 &vB)	// retourne le facteur de la projection de *this sur la droite passant par vA et vB
{
	CVector3 v = vB - vA;
	return v.Dot(*this - vA) / v.Dot(v);
}

CVector3 CVector3::Vector(CVector3 &vP)	// retourne le vecteur entre *this et vP
{
    return vP-*this;
}

//////////////////////////////
//	Les oprateurs simples	//
//////////////////////////////

void CVector3::operator = (CVector3 &v)
{
	this->xyz[0] = v.xyz[0];
	this->xyz[1] = v.xyz[1];
	this->xyz[2] = v.xyz[2];
}

void CVector3::operator = (float i)
{
	this->xyz[0] = i;
	this->xyz[1] = i;
	this->xyz[2] = i;
}

CVector3 CVector3::operator + (CVector3 &v)
{
	return CVector3(this->xyz[0]+v.xyz[0], this->xyz[1]+v.xyz[1], this->xyz[2]+v.xyz[2]);
}

CVector3 CVector3::operator + (float i)
{
	return CVector3(this->xyz[0]+i, this->xyz[1]+i, this->xyz[2]+i);
}

CVector3 CVector3::operator - (CVector3 &v)
{
	return CVector3(this->xyz[0]-v.xyz[0], this->xyz[1]-v.xyz[1], this->xyz[2]-v.xyz[2]);
}

CVector3 CVector3::operator - (float i)
{
	return CVector3(this->xyz[0]-i, this->xyz[1]-i, this->xyz[2]-i);
}

CVector3 CVector3::operator * (CVector3 &v)
{
	return CVector3(this->xyz[0]*v.xyz[0], this->xyz[1]*v.xyz[1], this->xyz[2]*v.xyz[2]);
}

CVector3 CVector3::operator * (float i)
{
	return CVector3(this->xyz[0]*i, this->xyz[1]*i, this->xyz[2]*i);
}

CVector3 CVector3::operator / (CVector3 &v)
{
	return CVector3(this->xyz[0]/v.xyz[0], this->xyz[1]/v.xyz[1], this->xyz[2]/v.xyz[2]);
}

CVector3 CVector3::operator / (float i)
{
	return CVector3(this->xyz[0]/i, this->xyz[1]/i, this->xyz[2]/i);
}

//////////////////////////////
//	Les oprateurs floats	//
//////////////////////////////

void CVector3::operator += (CVector3 &v)
{
	this->xyz[0] += v.xyz[0];
	this->xyz[1] += v.xyz[1];
	this->xyz[2] += v.xyz[2];
}

void CVector3::operator += (float i)
{
	this->xyz[0] += i;
	this->xyz[1] += i;
	this->xyz[2] += i;
}

void CVector3::operator -= (CVector3 &v)
{
	this->xyz[0] -= v.xyz[0];
	this->xyz[1] -= v.xyz[1];
	this->xyz[2] -= v.xyz[2];
}

void CVector3::operator -= (float i)
{
	this->xyz[0] -= i;
	this->xyz[1] -= i;
	this->xyz[2] -= i;
}

void CVector3::operator *= (CVector3 &v)
{
	this->xyz[0] *= v.xyz[0];
	this->xyz[1] *= v.xyz[1];
	this->xyz[2] *= v.xyz[2];
}

void CVector3::operator *= (float i)
{
	this->xyz[0] *= i;
	this->xyz[1] *= i;
	this->xyz[2] *= i;
}

void CVector3::operator /= (CVector3 &v)
{
	this->xyz[0] /= v.xyz[0];
	this->xyz[1] /= v.xyz[1];
	this->xyz[2] /= v.xyz[2];
}

void CVector3::operator /= (float i)
{
	this->xyz[0] /= i;
	this->xyz[1] /= i;
	this->xyz[2] /= i;
}

//////////////////////////////
//	Les oprateurs de test	//
//////////////////////////////

bool CVector3::operator == (CVector3 &v)
{
	return ((this->xyz[0]==v.xyz[0])&&(this->xyz[1]==v.xyz[1])&&(this->xyz[2]==v.xyz[2]));
}

bool CVector3::operator != (CVector3 &v)
{
	return ((this->xyz[0]!=v.xyz[0])||(this->xyz[1]!=v.xyz[1])||(this->xyz[2]!=v.xyz[2]));
}

