#ifndef __HCVector3__
#define __HCVector3__

typedef struct
{
	float xyz[3];
}SVector3;

class CVector3 {
public:
	float xyz[3];

	CVector3(void);	// constructeur par dfaut
	CVector3(CVector3 &v);	// constructeur avec duplication d'un vecteur
#ifdef __HCVector2__
	CVector3(CVector2 &v,float z);	// constructeur d'un CVector3  partir d'un CVector2 + coordonne Z
	CVector3(float z,CVector2 &v);	// constructeur d'un CVector3  partir d'une coordonne Z + CVector2
#endif
#ifdef __HCVector4__
    CVector3(CVector4 &v);	// constructeur d'un CVector3  partir d'un CVector4
#endif
    CVector3(float x,float y,float z);	// constructeur avec initialisation du vecteur
    ~CVector3(void);	// destructeur
    void Move(CVector3 &v);	// dplace le vecteur
    void Move(float x,float y,float z);	// dplace le vecteur
    void RotateXr(float angle);	// rotation du vecteur sur l'axe X d'un angle donn en radians
    void RotateYr(float angle);	// rotation du vecteur sur l'axe Y d'un angle donn en radians
    void RotateZr(float angle);	// rotation du vecteur sur l'axe Z d'un angle donn en radians
    void RotateX(float angle);	// rotation du vecteur sur l'axe X d'un angle donn en degrs
    void RotateY(float angle);	// rotation du vecteur sur l'axe Y d'un angle donn en degrs
    void RotateZ(float angle);	// rotation du vecteur sur l'axe Z d'un angle donn en degrs
    void Rotate(CVector3 &angle);	// rotation du vecteur sur les 3 axes XYZ d'un angle donn en degrs
    void Matrix(float *matrix); // traitement du vecteur avec une matrice 4x4
    float Length(void); // longueur du vecteur (aussi appel norme ou magnitude)
    float Distance(CVector3 &v);	// Distance entre 2 extrmits de vecteurs
    float Dot(CVector3 &v);	// Produit scalaire
    CVector3 Cross(CVector3 &v);	// Normale  2 vecteurs (produit vectoriel)
    CVector3 Linterpolate(CVector3 &u, CVector3 &v, float x);	// Fonction d'interpolation linaire entre 2 vecteurs
//    CVector3 Cinterpolate(CVector3 &u0, CVector3 &u1, CVector3 &u2, CVector3 &u3, float x);	// Fonction d'interpolation circulaire
    float AngleRad(CVector3 &v);	// retourne l'angle en radians entre le vecteur *this et le vecteur v2
    float AngleDeg(CVector3 &v2);	// retourne l'angle en degrs entre le vecteur *this et le vecteur v2
    void Normalize(void);	// normalise le vecteur
    CVector3 Normalized(void);	// retourne la norme de *this
    CVector3 ClosestPointOnLine(CVector3 &vA, CVector3 &vB);	// retourne les coordonne du point le plus proche de *this sur la droite passant par vA et vB
    CVector3 ClosestPointOnSegment(CVector3 &vA, CVector3 &vB);	// retourne les coordonne du point le plus proche de *this sur le segment vA,vB
    float ProjectionOnLine(CVector3 &vA, CVector3 &vB);	// retourne le facteur de la projection de *this sur la droite passant par vA et vB
    CVector3 Vector(CVector3 &vP);	// retourne le vecteur entre *this et vP

    void operator = (CVector3 &v);
    void operator = (float i);
    CVector3 operator + (CVector3 &v);
    CVector3 operator + (float i);
    CVector3 operator - (CVector3 &v);
    CVector3 operator - (float i);
    CVector3 operator * (CVector3 &v);
    CVector3 operator * (float i);
    CVector3 operator / (CVector3 &v);
    CVector3 operator / (float i);

    void operator += (CVector3 &v);
    void operator += (float i);
    void operator -= (CVector3 &v);
    void operator -= (float i);
    void operator *= (CVector3 &v);
    void operator *= (float i);
    void operator /= (CVector3 &v);
    void operator /= (float i);

    bool operator == (CVector3 &v);
    bool operator != (CVector3 &v);

    operator const float *() const { return xyz; }
    float &operator[](unsigned int index){ return *(((float *) xyz) + index); }
};

#endif // __HCVector3__
