#include <float.h>
#include <math.h>

#include "CVector2.h"
#include "CVector3.h"
#include "CVector4.h"
#include "CFastMaths.h"

CVector4::CVector4()	// constructeur par dfaut
{
	*this = 0;
	this->xyzw[3] = 1.0f;
}

CVector4::CVector4(CVector4 &v)	// constructeur avec duplication d'un vecteur
{
	*this = v;
}

#ifdef __HCVector2__
CVector4::CVector4(CVector2 &v,float z, float w)	// constructeur d'un CVector4  partir d'un CVector2 + coordonne Z
{
	this->xyzw[0] = v.xy[0];
	this->xyzw[1] = v.xy[1];
	this->xyzw[2] = z;
	this->xyzw[3] = w;
}
#endif

#ifdef __HCVector3__
CVector4::CVector4(CVector3 &v,float w)	// constructeur d'un CVector4  partir d'un CVector3 + coordonne W
{
	this->xyzw[0] = v.xyz[0];
	this->xyzw[1] = v.xyz[1];
	this->xyzw[2] = v.xyz[2];
	this->xyzw[3] = w;
}

CVector4::CVector4(float w,CVector3 &v)	// constructeur d'un CVector4  partir d'un coordonne W + CVector3
{
	this->xyzw[0] = v.xyz[0];
	this->xyzw[1] = v.xyz[1];
	this->xyzw[2] = v.xyz[2];
	this->xyzw[3] = w;
}
#endif

CVector4::CVector4(float x,float y,float z,float w)	// constructeur avec initialisation du vecteur
{
	this->xyzw[0] = x;
	this->xyzw[1] = y;
	this->xyzw[2] = z;
	this->xyzw[3] = w;
}

CVector4::~CVector4()	// destructeur
{
	// nothing todo
}

void CVector4::Move(CVector4 &v)	// dplace le vecteur
{
	*this += v;
}

void CVector4::Move(float x,float y,float z,float w)	// dplace le vecteur
{
	this->xyzw[0] += x;
	this->xyzw[1] += y;
	this->xyzw[2] += z;
	this->xyzw[3] += w;
}

void CVector4::RotateXr(float angle)	// rotation du vecteur sur l'axe X d'un angle donn en radians
{
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyzw[1]*ca-this->xyzw[2]*sa;
	this->xyzw[2]=this->xyzw[1]*sa+this->xyzw[2]*ca;
	this->xyzw[1]=tmp;
}

void CVector4::RotateYr(float angle)	// rotation du vecteur sur l'axe Y d'un angle donn en radians
{
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyzw[0]*ca+this->xyzw[2]*sa;
	this->xyzw[2]=this->xyzw[2]*ca-this->xyzw[0]*sa;
	this->xyzw[0]=tmp;
}

void CVector4::RotateZr(float angle)	// rotation du vecteur sur l'axe Z d'un angle donn en radians
{
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyzw[0]*ca-this->xyzw[1]*sa;
	this->xyzw[1]=this->xyzw[0]*sa+this->xyzw[1]*ca;
	this->xyzw[0]=tmp;
}

void CVector4::RotateX(float angle)	// rotation du vecteur sur l'axe X d'un angle donn en degrs
{
	angle=(angle/180.0f)*MYMATHS_PI;
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyzw[1]*ca-this->xyzw[2]*sa;
	this->xyzw[2]=this->xyzw[1]*sa+this->xyzw[2]*ca;
	this->xyzw[1]=tmp;
}

void CVector4::RotateY(float angle)	// rotation du vecteur sur l'axe Y d'un angle donn en degrs
{
	angle=(angle/180.0f)*MYMATHS_PI;
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyzw[0]*ca+this->xyzw[2]*sa;
	this->xyzw[2]=this->xyzw[2]*ca-this->xyzw[0]*sa;
	this->xyzw[0]=tmp;
}

void CVector4::RotateZ(float angle)	// rotation du vecteur sur l'axe Z d'un angle donn en degrs
{
	angle=(angle/180.0f)*MYMATHS_PI;
	float sa=(float)sin(angle);
	float ca=(float)cos(angle);
	float tmp=this->xyzw[0]*ca-this->xyzw[1]*sa;
	this->xyzw[1]=this->xyzw[0]*sa+this->xyzw[1]*ca;
	this->xyzw[0]=tmp;
}

void CVector4::Matrix(float *matrix) // traitement du vecteur avec une matrice 4x4
{
	float xtmp = (matrix[0]*this->xyzw[0]) + (matrix[4]*this->xyzw[1]) + (matrix[8]*this->xyzw[2]) + (matrix[12]*this->xyzw[3]);
	float ytmp = (matrix[1]*this->xyzw[0]) + (matrix[5]*this->xyzw[1]) + (matrix[9]*this->xyzw[2]) + (matrix[13]*this->xyzw[3]);
	float ztmp = (matrix[2]*this->xyzw[0]) + (matrix[6]*this->xyzw[1]) + (matrix[10]*this->xyzw[2]) + (matrix[14]*this->xyzw[3]);
	this->xyzw[3] = (matrix[3]*this->xyzw[0]) + (matrix[7]*this->xyzw[1]) + (matrix[11]*this->xyzw[2]) + (matrix[15]*this->xyzw[3]);
	this->xyzw[2] = ztmp;
	this->xyzw[1] = ytmp;
	this->xyzw[0] = xtmp;
}

float CVector4::Length() // longueur du vecteur (aussi appel norme ou magnitude)
{
	return (float) sqrt((this->xyzw[0]*this->xyzw[0]) + (this->xyzw[1]*this->xyzw[1]) + (this->xyzw[2]*this->xyzw[2]) + (this->xyzw[3]*this->xyzw[3]));
}

float CVector4::Distance(CVector4 &v)	// Distance entre 2 extrmits de vecteurs
{
	return (float)sqrt((v.xyzw[0]-this->xyzw[0])*(v.xyzw[0]-this->xyzw[0]) + (v.xyzw[1]-this->xyzw[1])*(v.xyzw[1]-this->xyzw[1]) + (v.xyzw[2]-this->xyzw[2])*(v.xyzw[2]-this->xyzw[2]) + (v.xyzw[3]-this->xyzw[3])*(v.xyzw[3]-this->xyzw[3]));
}

float CVector4::Dot(CVector4 &v)	// Produit scalaire
{
	return ((this->xyzw[0]*v.xyzw[0]) + (this->xyzw[1]*v.xyzw[1]) + (this->xyzw[2]*v.xyzw[2]) + (this->xyzw[3]*v.xyzw[3]));
}

CVector4 CVector4::Cross(CVector4 &v)	// Normale  2 vecteurs (produit vectoriel)
{
	return CVector4(this->xyzw[1]*v.xyzw[2]-v.xyzw[1]*this->xyzw[2], this->xyzw[2]*v.xyzw[3]-this->xyzw[3]*v.xyzw[2], this->xyzw[3]*v.xyzw[0]-this->xyzw[0]*v.xyzw[3], this->xyzw[0]*v.xyzw[1]-this->xyzw[1]*v.xyzw[0]);
}

CVector4 CVector4::Linterpolate (CVector4 &u, CVector4 &v, float x)	// Fonction d'interpolation linaire entre 2 vecteurs
{
	return u * (1 - x) + v * x;
}
/*
CVector4 CVector4::Cinterpolate (CVector4 &u0, CVector4 &u1, CVector4 &u2, CVector4 &u3, float x)	// Fonction d'interpolation circulaire
{
	CVector4 p = (u3 - u2) - (u0 - u1);
	CVector4 q = (u0 - u1) - p;
	CVector4 r = u2 - u0;
	return (((p * x + q) * x + r) * x + u1);
}
*/
float CVector4::AngleRad(CVector4 &v2)	// retourne l'angle en radians entre le vecteur *this et le vecteur v2
{                           
    float angle = (float)acos( this->Dot(v2) / this->Length() * v2.Length() );

    if(_isnan(angle))
        return 0;
    
    return angle;
}

float CVector4::AngleDeg(CVector4 &v2)	// retourne l'angle en degrs entre le vecteur *this et le vecteur v2
{                           
    float angle = (float)(180.0f * (acos( this->Dot(v2) / this->Length() * v2.Length() ) / MYMATHS_PI));

    if(_isnan(angle))
        return 0;
    
    return angle;
}

void CVector4::Normalize()	// normalise le vecteur
{
  float l = Length();

  if (l > 0.0f)
	*this /= l;
}

CVector4 CVector4::Normalized()	// retourne la norme de *this
{
	float l = Length();

	if (l > 0.0f)
	{
		*this /= l;
		return *this;
	}
	return CVector4(0,1,0,1);
}

CVector4 CVector4::ClosestPointOnLine(CVector4 &vA, CVector4 &vB)	// retourne les coordonne du point le plus proche de *this sur la droite passant par vA et vB
{
	return (((vB-vA) * ProjectionOnLine(vA, vB)) + vA);
}

CVector4 CVector4::ClosestPointOnSegment(CVector4 &vA, CVector4 &vB)	// retourne les coordonne du point le plus proche de *this sur le segment vA,vB
{
	float factor = ProjectionOnLine(vA, vB);
	if (factor<=0.0f)
		return vA;
	if (factor>=1.0f)
		return vB;
	return (((vB-vA) * factor) + vA);
}

float CVector4::ProjectionOnLine(CVector4 &vA, CVector4 &vB)	// retourne le facteur de la projection de *this sur la droite passant par vA et vB
{
	CVector4 v = vB - vA;
	return v.Dot(*this - vA) / v.Dot(v);
}

CVector4 CVector4::Vector(CVector4 &vP)	// retourne le vecteur entre *this et vP
{
    return vP-*this;
}

//////////////////////////////
//	Les oprateurs simples	//
//////////////////////////////

void CVector4::operator = (CVector4 &v)
{
	this->xyzw[0] = v.xyzw[0];
	this->xyzw[1] = v.xyzw[1];
	this->xyzw[2] = v.xyzw[2];
	this->xyzw[3] = v.xyzw[3];
}

void CVector4::operator = (float i)
{
	this->xyzw[0] = i;
	this->xyzw[1] = i;
	this->xyzw[2] = i;
	this->xyzw[3] = i;
}

CVector4 CVector4::operator + (CVector4 &v)
{
	return CVector4(this->xyzw[0]+v.xyzw[0], this->xyzw[1]+v.xyzw[1], this->xyzw[2]+v.xyzw[2], this->xyzw[3]+v.xyzw[3]);
}

CVector4 CVector4::operator + (float i)
{
	return CVector4(this->xyzw[0]+i, this->xyzw[1]+i, this->xyzw[2]+i, this->xyzw[3]+i);
}

CVector4 CVector4::operator - (CVector4 &v)
{
	return CVector4(this->xyzw[0]-v.xyzw[0], this->xyzw[1]-v.xyzw[1], this->xyzw[2]-v.xyzw[2], this->xyzw[3]-v.xyzw[3]);
}

CVector4 CVector4::operator - (float i)
{
	return CVector4(this->xyzw[0]-i, this->xyzw[1]-i, this->xyzw[2]-i, this->xyzw[3]-i);
}

CVector4 CVector4::operator * (CVector4 &v)
{
	return CVector4(this->xyzw[0]*v.xyzw[0], this->xyzw[1]*v.xyzw[1], this->xyzw[2]*v.xyzw[2], this->xyzw[3]*v.xyzw[3]);
}

CVector4 CVector4::operator * (float i)
{
	return CVector4(this->xyzw[0]*i, this->xyzw[1]*i, this->xyzw[2]*i, this->xyzw[3]*i);
}

CVector4 CVector4::operator / (CVector4 &v)
{
	return CVector4(this->xyzw[0]/v.xyzw[0], this->xyzw[1]/v.xyzw[1], this->xyzw[2]/v.xyzw[2], this->xyzw[3]/v.xyzw[3]);
}

CVector4 CVector4::operator / (float i)
{
	return CVector4(this->xyzw[0]/i, this->xyzw[1]/i, this->xyzw[2]/i, this->xyzw[3]/i);
}

//////////////////////////////
//	Les oprateurs floats	//
//////////////////////////////

void CVector4::operator += (CVector4 &v)
{
	this->xyzw[0] += v.xyzw[0];
	this->xyzw[1] += v.xyzw[1];
	this->xyzw[2] += v.xyzw[2];
	this->xyzw[3] += v.xyzw[3];
}

void CVector4::operator += (float i)
{
	this->xyzw[0] += i;
	this->xyzw[1] += i;
	this->xyzw[2] += i;
	this->xyzw[3] += i;
}

void CVector4::operator -= (CVector4 &v)
{
	this->xyzw[0] -= v.xyzw[0];
	this->xyzw[1] -= v.xyzw[1];
	this->xyzw[2] -= v.xyzw[2];
	this->xyzw[3] -= v.xyzw[3];
}

void CVector4::operator -= (float i)
{
	this->xyzw[0] -= i;
	this->xyzw[1] -= i;
	this->xyzw[2] -= i;
	this->xyzw[3] -= i;
}

void CVector4::operator *= (CVector4 &v)
{
	this->xyzw[0] *= v.xyzw[0];
	this->xyzw[1] *= v.xyzw[1];
	this->xyzw[2] *= v.xyzw[2];
	this->xyzw[3] *= v.xyzw[3];
}

void CVector4::operator *= (float i)
{
	this->xyzw[0] *= i;
	this->xyzw[1] *= i;
	this->xyzw[2] *= i;
	this->xyzw[3] *= i;
}

void CVector4::operator /= (CVector4 &v)
{
	this->xyzw[0] /= v.xyzw[0];
	this->xyzw[1] /= v.xyzw[1];
	this->xyzw[2] /= v.xyzw[2];
	this->xyzw[3] /= v.xyzw[3];
}

void CVector4::operator /= (float i)
{
	this->xyzw[0] /= i;
	this->xyzw[1] /= i;
	this->xyzw[2] /= i;
	this->xyzw[3] /= i;
}

//////////////////////////////
//	Les oprateurs de test	//
//////////////////////////////

bool CVector4::operator == (CVector4 &v)
{
	return ((this->xyzw[0]==v.xyzw[0])&&(this->xyzw[1]==v.xyzw[1])&&(this->xyzw[2]==v.xyzw[2])&&(this->xyzw[3]==v.xyzw[3]));
}

bool CVector4::operator != (CVector4 &v)
{
	return ((this->xyzw[0]!=v.xyzw[0])||(this->xyzw[1]!=v.xyzw[1])||(this->xyzw[2]!=v.xyzw[2])||(this->xyzw[3]!=v.xyzw[3]));
}

