#ifndef __HCVector4__
#define __HCVector4__

typedef struct
{
	float xyzw[4];
}SVector4;

class CVector4 {
public:
	float xyzw[4];

	CVector4(void);	// constructeur par dfaut
	CVector4(CVector4 &v);	// constructeur avec duplication d'un vecteur
#ifdef __HCVector2__
	CVector4(CVector2 &v,float z,float w);	// constructeur d'un CVector4  partir d'un CVector2 + coordonne Z et W
#endif
#ifdef __HCVector3__
	CVector4(CVector3 &v,float w);	// constructeur d'un CVector4  partir d'un CVector3 + coordonne W
	CVector4(float w,CVector3 &v);	// constructeur d'un CVector4  partir d'une coordonne W + CVector3
#endif
	CVector4(float x,float y,float z,float w);	// constructeur avec initialisation du vecteur
	~CVector4(void);	// destructeur
	void Move(CVector4 &v);	// dplace le vecteur
	void Move(float x,float y,float z,float w);	// dplace le vecteur
	void RotateXr(float angle);	// rotation du vecteur sur l'axe X d'un angle donn en radians
	void RotateYr(float angle);	// rotation du vecteur sur l'axe Y d'un angle donn en radians
	void RotateZr(float angle);	// rotation du vecteur sur l'axe Z d'un angle donn en radians
	void RotateX(float angle);	// rotation du vecteur sur l'axe X d'un angle donn en degrs
	void RotateY(float angle);	// rotation du vecteur sur l'axe Y d'un angle donn en degrs
	void RotateZ(float angle);	// rotation du vecteur sur l'axe Z d'un angle donn en degrs
	void Matrix(float *matrix); // traitement du vecteur avec une matrice 4x4
	float Length(void); // longueur du vecteur (aussi appel norme ou magnitude)
	float Distance(CVector4 &v);	// Distance entre 2 extrmits de vecteurs
	float Dot(CVector4 &v);	// Produit scalaire
	CVector4 Cross(CVector4 &v);	// Normale  2 vecteurs (produit vectoriel)
	CVector4 Linterpolate(CVector4 &u, CVector4 &v, float x);	// Fonction d'interpolation linaire entre 2 vecteurs
//	CVector4 Cinterpolate(CVector4 &u0, CVector4 &u1, CVector4 &u2, CVector4 &u3, float x);	// Fonction d'interpolation circulaire
	float AngleRad(CVector4 &v);	// retourne l'angle en radians entre le vecteur *this et le vecteur v2
	float AngleDeg(CVector4 &v2);	// retourne l'angle en degrs entre le vecteur *this et le vecteur v2
	void Normalize(void);	// normalise le vecteur
	CVector4 Normalized(void);	// retourne la norme de *this
	CVector4 ClosestPointOnLine(CVector4 &vA, CVector4 &vB);	// retourne les coordonne du point le plus proche de *this sur la droite passant par vA et vB
	CVector4 ClosestPointOnSegment(CVector4 &vA, CVector4 &vB);	// retourne les coordonne du point le plus proche de *this sur le segment vA,vB
	float ProjectionOnLine(CVector4 &vA, CVector4 &vB);	// retourne le facteur de la projection de *this sur la droite passant par vA et vB
	CVector4 Vector(CVector4 &vP);	// retourne le vecteur entre *this et vP

	void operator = (CVector4 &v);
	void operator = (float i);
	CVector4 operator + (CVector4 &v);
	CVector4 operator + (float i);
	CVector4 operator - (CVector4 &v);
	CVector4 operator - (float i);
	CVector4 operator * (CVector4 &v);
	CVector4 operator * (float i);
	CVector4 operator / (CVector4 &v);
	CVector4 operator / (float i);

	void operator += (CVector4 &v);
	void operator += (float i);
	void operator -= (CVector4 &v);
	void operator -= (float i);
	void operator *= (CVector4 &v);
	void operator *= (float i);
	void operator /= (CVector4 &v);
	void operator /= (float i);

	bool operator == (CVector4 &v);
	bool operator != (CVector4 &v);

	operator const float *() const { return xyzw; }
	float &operator[](unsigned int index){ return *(((float *) xyzw) + index); }
};

#endif // __HCVector4__
