#include <windows.h>
#include <stdlib.h>
#include <float.h>
#include <stdio.h>
#include <math.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include "CVector3.h"
#include "CVector4.h"
#include "CMatrix4.h"
#include "CMaterial.h"
#include "CObjet3D.h"
#include "CCamera.h"
#include "CTexture.h"
#include "CTerrain.h"
#include "CSkyBox.h"
#include "NormalizePlane.h"
#include "Enums.h"
#include "MainLoop.h"

#define MAX_NB_OBJETS3D	3
#define NB_OBJETS3D		3

CObjet3D *objets3D[MAX_NB_OBJETS3D];		// les objets
CVector3 objets_angle[MAX_NB_OBJETS3D];		// orientation de chaque objet
CVector3 objets_position[MAX_NB_OBJETS3D];	// position de chaque objet
CVector3 vLight(-1.0f, 1.8f, -0.5f);		// la position de la lumire
CCamera *camera;
CMaterial *materials;
CTexture *textures;
CTerrain *terrain;
CSkyBox *skybox;

void Lumiere (CVector3& vLumiere)
{
	static CVector4 vAmbient(0.2f,0.2f,0.2f,1.0f);
	static CVector4 vDiffuse(1.0f,1.0f,1.0f,1.0f);
	static CVector4 vSpecular(0.1f,0.1f,0.1f,1.0f);
	CVector4 vPosition(vLumiere, 1);

	glLightfv(GL_LIGHT1,  GL_AMBIENT,	vAmbient.xyzw);
	glLightfv(GL_LIGHT1,  GL_DIFFUSE,	vDiffuse.xyzw);
	glLightfv(GL_LIGHT1,  GL_SPECULAR,	vSpecular.xyzw);
	glLightfv(GL_LIGHT1,  GL_POSITION,	vPosition.xyzw);
}

void Afficher ()
{
	int nbobj = 0;

	// On dessine une skybox donc on n'a pas besoin de vider le buffer de couleur
	glClear (GL_DEPTH_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);
	glLoadIdentity ();

	camera->Draw();
	vLight.RotateY(0.33f);
	Lumiere(vLight);

	glBegin(GL_LINES);
		glVertex3fv(vLight.xyz);
		glVertex3f(0.0f,0.0f,0.0f);
		glVertex3f(0.0f,0.0f,0.0f);
		glVertex3f(vLight.xyz[X],0.0f,vLight.xyz[Z]);
		glVertex3f(vLight.xyz[X],0.0f,vLight.xyz[Z]);
		glVertex3fv(vLight.xyz);
	glEnd();
	
// dessin des objets
	for(int nObjNb=0; nObjNb < NB_OBJETS3D ;nObjNb++)
	{
		glPushMatrix ();
			glTranslatef(objets_position[nObjNb].xyz[X], objets_position[nObjNb].xyz[Y], objets_position[nObjNb].xyz[Z]);
			glRotatef(objets_angle[nObjNb].xyz[Y], 0,1,0);
			objets3D[nObjNb]->Draw(); // dessin de l'objet 3D
		glPopMatrix ();
	}
	terrain->Draw(100.0f);
// dessin de l'ombre dans le stencil
	glStencilFunc(GL_ALWAYS, 1, ~0); // le stencil enregistre les pixels
	glEnable(GL_STENCIL_TEST); // activation du stencil
	// mettez la ligne suivante en commentaire pour voir les volumes (en mettant juste cette ligne en commentaire vous ne verrez que l'arrire des volumes, le cullfacing supprimera l'avant)
	glColorMask(0,0,0,0); // dsactivation de l'criture des pixels (on crit juste dans le stencil)
	glDepthMask(0); // on dsactive l'criture (pas les tests) dans le zbuffer
	for(int nObjNb=0; nObjNb < NB_OBJETS3D ;nObjNb++)
	{
		CVector3 vLightInv(vLight); // on cre un vecteur lumire que l'on tourne et positionne  l'inverse des objets
		vLightInv.RotateY(-objets_angle[nObjNb].xyz[Y]);
		vLightInv.Move(-objets_position[nObjNb].xyz[X], -objets_position[nObjNb].xyz[Y], -objets_position[nObjNb].xyz[Z]);
		glPushMatrix ();
			glTranslatef(objets_position[nObjNb].xyz[X], objets_position[nObjNb].xyz[Y], objets_position[nObjNb].xyz[Z]);
			glRotatef(objets_angle[nObjNb].xyz[Y], 0,1,0);
			glCullFace(GL_BACK); // mode face arrires
			glStencilOp(GL_KEEP, GL_KEEP, GL_INCR); // incrmentation du stencil
			objets3D[nObjNb]->DrawShadowVolume(vLightInv); // dessin du volume d'ombre arrire
			glCullFace(GL_FRONT); // mode faces avant
			glStencilOp(GL_KEEP, GL_KEEP, GL_DECR); // dcrmentation du stencil
			objets3D[nObjNb]->DrawShadowVolume(vLightInv); // dessin du volume d'ombre avant
			glCullFace(GL_BACK);
		glPopMatrix ();
	}
	glDepthMask(1); // on ractive l'criture dans le zbuffer
	glColorMask(1,1,1,1); // on ractive le dessin
	glDisable(GL_STENCIL_TEST);  // on dsactive le stencil

// dessin de l'ombre, un rectangle semi transparent recouvrant l'cran
	glColor4f(0.0f, 0.0f, 0.0f, 0.5f); // on choisit une couleur sombre (la noir, y a pas plus sombre ^__^) avec un peu de transparence en vue de superposer +sieurs ombres venant de sources lumineuses diffrentes comme dans la ralit, l o elles se superposent l'ombre s'assombrit
	glEnable(GL_BLEND); // activation des calculs de blending pour la transparence
	glStencilFunc(GL_NOTEQUAL, 0, ~0); // fonction de test du stencil
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP); // opration  effectuer
	glPushMatrix();
	glLoadIdentity(); // on remet les matrices  0
	glEnable(GL_STENCIL_TEST); // ractivation du stencil
	glBegin(GL_QUADS); // dessin d'un rectangle recouvrant toute la vue
		glVertex3f(-1.1f, 1.1f,-0.5f);
		glVertex3f(-1.1f,-1.1f,-0.5f);
		glVertex3f( 1.1f,-1.1f,-0.5f);
		glVertex3f( 1.1f, 1.1f,-0.5f);
	glEnd();
	glDisable(GL_STENCIL_TEST); // on peut  nouveau dsactiver le stencil
	glPopMatrix();
	glDisable(GL_BLEND); // et les effets de transparence

// dessin de la skybox
	skybox->Draw(100.0f);

//	objets_angle[1].xyz[Y] += 0.3f; // effetue une rotation de l'objet
}

void Initialisation (int xscr,int yscr)
{
	CIndice2 ecran(xscr,yscr);
	CVector3 Cpos( -0.5f, 2.8f, 3.0f);
	CVector3 Clook( 0.0f, 0.0f, 0.0f);
	textures = new CTexture();
	materials = new CMaterial();
	camera = new CCamera(ecran,Cpos,Clook);
	terrain = new CTerrain();
	objets3D[0] = new CObjet3D("miki.obj");
	objets3D[1] = new CObjet3D("tank.obj");
	objets3D[2] = new CObjet3D("pompiers.obj");
	skybox = new CSkyBox();

	srand(GetTickCount());
	objets_angle[0].xyz[Y] = -45.0f;
	objets_angle[1].xyz[Y] = 0.0f;
	objets_position[0].xyz[Y] = 0.8f;
	objets_position[0].xyz[Z] = 0.0f;
	objets_position[1].xyz[X] = -0.8f;
	objets_position[1].xyz[Z] = 0.0f;
	objets_position[2].xyz[X] = 0.8f;
	objets_position[2].xyz[Z] = 0.0f;

	glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
	glClearStencil(0);
	glShadeModel (GL_SMOOTH);
	glEnable (GL_CULL_FACE);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
	glEnable(GL_LIGHT1);
	glEnable (GL_LIGHTING);
	glEnable (GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	glEnable(GL_COLOR_MATERIAL);
	glLineWidth(3.0f);
	glPointSize(5.0f);
}

void Quitter ()
{
	for(int i=0; i < NB_OBJETS3D ;i++)
		delete objets3D[i];
	delete textures;
	delete materials;
	delete camera;
	delete terrain;
	delete skybox;
}
